/**
  ******************************************************************************
  * File Name          : main.c
  * Description        : Main program body
  ******************************************************************************
  *
  * COPYRIGHT(c) 2016 STMicroelectronics
  *
  * Redistribution and use in source and binary forms, with or without modification,
  * are permitted provided that the following conditions are met:
  *   1. Redistributions of source code must retain the above copyright notice,
  *      this list of conditions and the following disclaimer.
  *   2. Redistributions in binary form must reproduce the above copyright notice,
  *      this list of conditions and the following disclaimer in the documentation
  *      and/or other materials provided with the distribution.
  *   3. Neither the name of STMicroelectronics nor the names of its contributors
  *      may be used to endorse or promote products derived from this software
  *      without specific prior written permission.
  *
  * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
  * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
  * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
  * DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
  * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
  * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
  * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
  * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
  * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
  ******************************************************************************
  */
/* Includes ------------------------------------------------------------------*/
#include "main.h"
#include "stm32f1xx_hal.h"
#include "dac.h"
#include "tim.h"
#include "sound.h"
#include "maze.h"
#include "usart.h"
#include "gpio.h"
#include "ili9325_lcd_gpio_driver.h"

void SystemClock_Config(void);
void Error_Handler(void);
/*our code starts here*/
void GPIO_Configuration(void);
void USART_Configuration(void);
JOYState_TypeDef J;
void LCD_Configuration(void);
void Port_Config(void);                         //Configuring uController Ports
void move_badguys(void);                        //Moves bad guys based on Maze#
void User_Input(void);                          //Reads joystick direction
void check_movement(void);                      //Checks for possible movements
void condition(void);                           //Checks player's current condition
void save_maze(unsigned short n);           //Stores the current maze in usable array
void score(void);                               //Updates player score based on conditions
void move_all(void);                            //Manages player movement through maze
void Write_HighScore(unsigned short value); //Writes High Score to Flash Memory
unsigned short Read_HighScore(int addr);    //Reads and returns High Score from Flash Memory
void Write_Mem(int addr, short data);       //Writes value to Flash Memory
void Instructions(void); 
unsigned int high_score = 100;
unsigned int clk=0;
unsigned int i=0;
unsigned int average;
unsigned short x = 0;
uint32_t y = 0;
unsigned short count = 0;
unsigned short count2 = 0;
unsigned short badcount = 0;
unsigned short move;
unsigned short game_play=1;
unsigned short player_x, player_y, start_x, start_y;
unsigned short left,right,up,down,b_left,b_right,b_up,b_down;
unsigned short WIN = 0;
unsigned short LOSE = 0;
unsigned short SCORE = 0;
unsigned short LEVEL = 1;
unsigned int game_score = 300;
unsigned char maze[15][25];      
/*our code ends here*/



int main(void)
{
	/*our code begins here*/
uint8_t char_int[4];
 uint16_t i=4;
 uint16_t temp;
 uint16_t value;
	/*our code ends here*/
 J=BSP_JOY_GetState();
  HAL_Init();
  SystemClock_Config();
  MX_GPIO_Init();
  MX_USART1_UART_Init();
  MX_DAC_Init();
  MX_TIM3_Init();
	MX_TIM4_Init();
	LCD_Initializtion();
	//MX_DAC_Init();
	Port_Config();
  
	HAL_TIM_Base_Start(&htim3);
	
	
	//HAL_TIM_Base_Start_IT(&htim4);
	HAL_DAC_Start(&hdac, DAC_CHANNEL_2);
	LCD_Clear(GREEN);
	high_score = Read_HighScore(0x0807F800);
    Draw_Title(BLUE, WHITE, RED);
	temp = high_score;
	for(i=4;i>0;i--)
	{
	  value = temp % 10;
	  char_int[i-1] = value + 48;
	  temp = temp/10;
	}
    GUI_Text(150,200,char_int,BLUE,&Font8,WHITE);
  
	Mario_Main();
	Instructions();
	Draw_Maze1(BLACK, WHITE, BLUE);
	b_left=0;
	b_right=0;
	b_up=0;
	b_down=0;
	left=0;
	right=0;
	up=0;
	down=0;
	
	//TIM3->CR1|=0x1;
	HAL_TIM_Base_Start_IT(&htim3);
	HAL_TIM_Base_Start_IT(&htim4);
	while(game_play==1)
	{
	  Mario_Underground();
	}
    if(game_play==2)
    {
      Win_Sound();
    }
	
 
  while (1)
  {
Delay_ms(100);

  }
 

}
/*our code starts here*/
void Port_Config()
{
	__HAL_RCC_GPIOA_CLK_ENABLE();
__HAL_RCC_GPIOC_CLK_ENABLE();
	__HAL_RCC_TIM3_CLK_ENABLE();
__HAL_RCC_TIM4_CLK_ENABLE();
__HAL_RCC_DAC_CLK_ENABLE();
 HAL_NVIC_EnableIRQ(TIM3_IRQn);
	HAL_NVIC_EnableIRQ(TIM4_IRQn);
    
    AFIO->EXTICR[0] = 0x2000;
    HAL_NVIC_EnableIRQ(EXTI0_IRQn);
	HAL_NVIC_EnableIRQ(EXTI1_IRQn);
	HAL_NVIC_EnableIRQ(EXTI2_IRQn);
	HAL_NVIC_EnableIRQ(EXTI3_IRQn);
	save_maze(1);
    
    player_x=1;
    player_y=1;
	start_x=1;
	start_y=1;
	//J=BSPGRTSTATE();
	
	
}
void Instructions()
{
	LCD_Clear(GREEN);
	LCD_Box(BLUE, 20, 20, 5, 5);
	GUI_Text(30,20,(uint8_t *) "This is you.", BLACK,&Font12, WHITE);
	Delay_ms(2500);
	LCD_Box(RED, 20, 40, 5, 5);
	GUI_Text(30,40,(uint8_t *) "This is a badguy.", BLACK,&Font12, WHITE);
	GUI_Text(30,60,(uint8_t *) "You'll want to avoid these", BLACK,&Font12, WHITE);
	GUI_Text(30,80,(uint8_t *) "You'll lose points if you don't!", BLACK,&Font12,WHITE);
	Delay_ms(2500);
	GUI_Text(20,100,(uint8_t *) "*", BLUE,&Font12, WHITE);
	GUI_Text(30,100,(uint8_t *) "These are extra points!", BLACK,&Font12, WHITE);
	Delay_ms(2500);
	GUI_Text(20,120,(uint8_t *) "Don't get to 0 points or you'll lose!", BLACK,&Font12, WHITE);
	Delay_ms(2500);
	GUI_Text(20,140,(uint8_t *) "Try and win! Good luck!", BLACK,&Font12, WHITE);
	Delay_ms(3000);
}
void Write_HighScore(unsigned short value)
{
    //Initialize Memory
    unsigned short loop = 1;
    while(loop == 1)
    {
      FLASH->KEYR = 0x45670123; //Key1
      FLASH->KEYR = 0xCDEF89AB; //Key2
      if(((FLASH->CR)&0x80) == 0)
      {
        if(((FLASH->SR)&0x1) == 0)
        {
          FLASH->CR |= 0x2;
          FLASH->AR = 0x0807F800;
          FLASH->CR |= 0x40;
          while(((FLASH->SR)&0x1) == 1)
          {}
          loop = 0;
        }
      }
    }
    FLASH->CR &= 0xFFFFFFFD;
    FLASH->CR |= 0x1;
    while(((FLASH->SR)&0x1) == 1)
    {
    }
    Write_Mem(0x0807F800, value);
    FLASH->CR &= 0xFFFFFFFE;
}
unsigned short Read_HighScore(int addr)
{
    unsigned short data;
    __asm
    {
      ldrh data, [addr]
    }
    return data;
	}

//Write to Flash Memory:
// - ASM file
// - Write data value to address
void Write_Mem(int addr, short data)
{
    __asm
    {
      strh data, [addr]
    }
}
///*OUR CODE*/

//HAL_GPIO_EXTI_IRQHandler(UP_Pin);
//HAL_GPIO_EXTI_IRQHandler(DOWN_Pin);
void EXTI0_IRQHandler(void)
{HAL_GPIO_EXTI_IRQHandler(LEFT_Pin);
	//LCD_Clear(GREEN);
    User_Input();
    EXTI->PR=0x1;//LCD_Clear(GREEN);
} 


void EXTI1_IRQHandler(void)
{HAL_GPIO_EXTI_IRQHandler(UP_Pin);
	//LCD_Clear(BLUE);
    User_Input();
    SET_BIT(EXTI->PR,0x2);//LCD_Clear(BLUE);
}

void EXTI2_IRQHandler(void)
{HAL_GPIO_EXTI_IRQHandler(DOWN_Pin);
	//LCD_Clear(RED);
    User_Input();
    SET_BIT(EXTI->PR,0x4);//LCD_Clear(RED);
}

void EXTI3_IRQHandler(void)
{HAL_GPIO_EXTI_IRQHandler(RIGHT_Pin);
	
    User_Input();
    SET_BIT(EXTI->PR,0x8);//LCD_Clear(YELLOW);
}
void TIM3_IRQHandler()
{ 
HAL_TIM_IRQHandler(&htim3);
	//LCD_Clear(GREEN);
	if(count<50)
	{
	  count++;
	  badcount++;
	}
	else
	{
      LCD_Box(WHITE,30+((player_x-1)*10),60+((player_y-1)*10),10,10);
	  check_movement();
	  Delay_ms(0);
	  condition();
	  Delay_ms(0);
	  move_all();
	  b_left=0;
	  b_right=0;
	  b_up=0;
	  b_down=0;
	  left=0;
	  right=0;
	  up=0;
	  down=0;
	  if(WIN==1)
      {
        score();
        WIN = 0;
      }
      LCD_Box(BLUE,32+((player_x-1)*10),62+((player_y-1)*10),5,5);
	  if(badcount>200)
	  {
	    move_badguys();
	    badcount=0;
	  }
	  else
	  {
	    badcount++;
	  }
	  count = 0;
	}
	if(WIN==2)
	{
	  uint8_t char_int[4];
	  unsigned int i=4;
	  unsigned int temp = game_score;
	  char value;
	  LCD_Clear(WHITE);
	  for(i=4;i>0;i--)
	  {
		value = temp % 10;
		char_int[i-1] = value + 48;
		temp = temp/10;
	  }
      SCORE = 0;
      if(game_score>high_score)
      {
        Write_HighScore(game_score);
        high_score = game_score;
      }
      
      Draw_Win(YELLOW, BLUE, WHITE);
	  GUI_Text(150,160,(uint8_t *) char_int,WHITE,&Font8,BLUE);
      temp = high_score;
      for(i=4;i>0;i--)
      {
        value = temp % 10;
        char_int[i-1] = value + 48;
        temp = temp/10;
      }
      GUI_Text(150,190,(uint8_t *)char_int,WHITE,&Font8,BLUE);
      TIM3->CR1=0x10;
      game_play = 2;
	}
	TIM3->SR=(TIM3->SR&0xFFFFFFFE);
}

void TIM4_IRQHandler()
{HAL_TIM_IRQHandler(&htim4);
	if(y==0)
	{
		y=50;
	}
	else
	{
		y=0;
	}
	TIM4->SR=(TIM4->SR&0xFFFFFFFE);
	HAL_DAC_SetValue(&hdac, DAC_CHANNEL_2, DAC_ALIGN_12B_R, y);

    DAC->DHR12R2 = y;
}

void move_badguys()
{
	if(LEVEL == 1)	   //Maze 1: 2 bad guys
	{
		if(b1 ==10)
			move = 0;
		else
		{ if(b1 == 3)//3
			  move = 1;
		  else
			  move = move;
		}
		maze[b1][1] = ' ';
		maze[13][b2] = ' ';
		LCD_Box(WHITE,32,62+((b1)*10),5,5);//32
		LCD_Box(WHITE,22+((b2)*10),182,5,5);//22
		if(move == 1)  //Increment bad guy location
		{
			b1++;
			b2++;
		}
		else           //Decrement bad guy location
		{
			b1--;
			b2--;
		}
		maze[b1][1] = 'b';
		maze[13][b2] = 'b';
		LCD_Box(RED,32,62+(b1*10),5,5);
		LCD_Box(RED,22+(b2*10),182,5,5);
	}
}

void check_movement()
{
	Delay_ms(0);
    if(maze[player_y-1][player_x]=='#')
	  up = 0;
	else
	  up = 1;
	if(maze[player_y+1][player_x]=='#')
	  down = 0;
	else
	  down = 1;
	if(maze[player_y][player_x-1]=='#')
	  left = 0;
	else
	  left = 1;
	if(maze[player_y][player_x+1]=='#')
	  right = 0;
	else
	  right = 1;
}


void User_Input()
{
    //unsigned short input = (GPIOC->IDR&0x8)+(GPIOA->IDR&0x7);
	JOYState_TypeDef input=BSP_JOY_GetState();
    if(input==DOWN_Pin)
    {//LCD_Clear(YELLOW);
	  b_down=1;
    }
    else if(input==LEFT_Pin)
    {//LCD_Clear(BLACK);
	  b_left=1;
    }
    else if(input==RIGHT_Pin)
    {//LCD_Clear(RED);
	  b_right=1;
    }
    else if(input==UP_Pin)
    {//LCD_Clear(BLUE);
	  b_up=1;
    }
}

//Save Current Maze:
// - Stores current maze into an editable
//   2D char array that is accessed for 
//   movement and status
void save_maze(unsigned short n)
{
	unsigned short i,j;
	if(n==1)
	{
	  for(i=0;i<15;)
	  {
		for(j=0;j<25;)
		{
		  maze[i][j] = Maze1[i][j];
		  Delay_ms(2);
		  j++;
		}
		i++;
	  }
	}
	if(n==2)
	{
	  for(i=0;i<15;)
	  {
		for(j=0;j<25;)
		{
		  maze[i][j] = Maze2[i][j];
		  j++;
		}
		i++;
	  }
	  player_x--;
	}
	if(n==3)
	{
	  for(i=0;i<15;)
	  {
		for(j=0;j<25;)
		{
		  maze[i][j] = Maze3[i][j];
		  j++;
		}
		i++;
	  }
	}
		
}

//Player Condition Function:
// - Checks to see what condition the player
//   is currently at in the maze
//   'e' - end of level, go to next level
//   'b' - bad guy, lose condition
//   'p' - pit, lose condition
//   '$' - points
void condition()
{
	if(maze[player_y][player_x] == 'e')
	{
	  WIN = 1;
      LEVEL++;
	}
	else if(maze[player_y][player_x] == 'b')
	  LOSE = 1;
	else if(maze[player_y][player_x] == 'p')
	  LOSE = 1;
	else if(maze[player_y][player_x] == '$')
	{
	  SCORE = 1;
	  maze[player_y][player_x] = ' ';
	}
}

//Move Player Function:
// - If player hasn't won or lost
//   move the player based on movements
// - If points drop below 50,
//   * Lose screen goes to LCD
//   * Movement timer is halted
// - If player wins a level
//   * Save next maze to memory
//   * Change bad guy locations
//   * Change start position
// - If player wins level 3
//   * Compare score to high score
//   * Movement timer is halted
//   * Give win condition
void move_all()
{
	if(game_score>50)
	{
	  if(LOSE==0)
	  {
		if(WIN==0)
		{
		  if(b_left==1)
		  {
			if(left==1)
			{
				player_x--;
			}
		  }
		  else if(b_right==1)
		  {
            if(right==1)
			{
			  player_x++;
			}
		  }
		  else if(b_up==1)
		  {
			if(up==1)
			{
			  player_y--;
			}
		  }
		  else if(b_down==1)
		  {
			if(down==1)
			{
			  player_y++;
			}
		  }
		  score();
		}
		else
		{
		  save_maze(LEVEL);
		  if(LEVEL==2)
		  {
			Draw_Maze2(BLACK,WHITE,BLUE);
            start_x = player_x;
            start_y = player_y;
		  }
		  else if(LEVEL==3)
		  {
			Draw_Maze3(BLACK,WHITE,BLUE);
            start_x = player_x-1;
            start_y = player_y;
		  }
          else if(LEVEL>3)
          {
            WIN = 2;
            score();
            game_play = 2;
          }          
		}
	  }
	  else
	  {
		player_x=start_x;
		player_y=start_y;
		score();
		LOSE = 0;
	  }
	}
	else
	{
      Draw_Lose(RED, BLACK, BLACK);
	  TIM3->CR1=0x10;
      game_play = 0;
	}
}


void score()
{
    uint8_t char_int[4];
	unsigned int i=4;
	unsigned int temp;
	char value;
	
    if(WIN==1)
      game_score += 500;
    else if(WIN==2)
	  game_score += 1000;
    else if(LOSE==1)
      game_score -= 100;
    else if(SCORE==1)
      game_score += 250;
    temp = game_score;
	for(i=4;i>0;i--)
	{
	  value = temp % 10;
	  char_int[i-1] = value + 48;
	  temp = temp/10;
	}
	SCORE = 0;
	GUI_Text(100,20,(uint8_t *)"Score",BLACK,&Font8,WHITE);
	GUI_Text(150,20,char_int,BLACK,&Font8,WHITE);
}

/*our code ends here*/
void SystemClock_Config(void)
{

  RCC_OscInitTypeDef RCC_OscInitStruct;
  RCC_ClkInitTypeDef RCC_ClkInitStruct;
  RCC_OscInitStruct.OscillatorType = RCC_OSCILLATORTYPE_HSI;
  RCC_OscInitStruct.HSIState = RCC_HSI_ON;
  RCC_OscInitStruct.HSICalibrationValue = 16;
  RCC_OscInitStruct.PLL.PLLState = RCC_PLL_NONE;
  if (HAL_RCC_OscConfig(&RCC_OscInitStruct) != HAL_OK)
  {
    Error_Handler();
  }

    /**Initializes the CPU, AHB and APB busses clocks 
    */
  RCC_ClkInitStruct.ClockType = RCC_CLOCKTYPE_HCLK|RCC_CLOCKTYPE_SYSCLK
                              |RCC_CLOCKTYPE_PCLK1|RCC_CLOCKTYPE_PCLK2;
  RCC_ClkInitStruct.SYSCLKSource = RCC_SYSCLKSOURCE_HSI;
  RCC_ClkInitStruct.AHBCLKDivider = RCC_SYSCLK_DIV1;
  RCC_ClkInitStruct.APB1CLKDivider = RCC_HCLK_DIV2;
  RCC_ClkInitStruct.APB2CLKDivider = RCC_HCLK_DIV1;

  if (HAL_RCC_ClockConfig(&RCC_ClkInitStruct, FLASH_LATENCY_0) != HAL_OK)
  {
    Error_Handler();
  }

    /**Configure the Systick interrupt time 
    */
  HAL_SYSTICK_Config(HAL_RCC_GetHCLKFreq()/1000);

    /**Configure the Systick 
    */
  HAL_SYSTICK_CLKSourceConfig(SYSTICK_CLKSOURCE_HCLK);

  /* SysTick_IRQn interrupt configuration */
  HAL_NVIC_SetPriority(SysTick_IRQn, 0, 0);
}

/* USER CODE BEGIN 4 */

/* USER CODE END 4 */

/**
  * @brief  This function is executed in case of error occurrence.
  * @param  None
  * @retval None
  */
void Error_Handler(void)
{
  /* USER CODE BEGIN Error_Handler */
  /* User can add his own implementation to report the HAL error return state */
  while(1) 
  {
  }
  /* USER CODE END Error_Handler */ 
}

#ifdef USE_FULL_ASSERT

/**
   * @brief Reports the name of the source file and the source line number
   * where the assert_param error has occurred.
   * @param file: pointer to the source file name
   * @param line: assert_param error line source number
   * @retval None
   */
void assert_failed(uint8_t* file, uint32_t line)
{
  }

#endif

/**
  * @}
  */ 

/**
  * @}
*/ 

/************************ (C) COPYRIGHT STMicroelectronics *****END OF FILE****/
